# Generated by roxygen2: do not edit by hand

export(analytiskontodimas_2004)
export(ashrafi1_2018)
export(ashrafi2_2018)
export(ashrafi3_2018)
export(ashrafi4_2018)
export(ashrafi5_2018)
export(atkin_2005)
export(beta_2012)
export(betatypesimplified_2008)
export(boatman_2017)
export(briere1_1999)
export(briere1simplified_1999)
export(briere2_1999)
export(briere2simplified_1999)
export(briereextended_2021)
export(briereextendedsimplified_2021)
export(calc_params)
export(delong_2017)
export(deutsch_2008)
export(eubank_1973)
export(flextpc_2024)
export(flinn_1991)
export(gaussian_1987)
export(gaussianmodified_2006)
export(get_breadth)
export(get_ctmax)
export(get_ctmin)
export(get_e)
export(get_eh)
export(get_lower_lims)
export(get_model_names)
export(get_q10)
export(get_rmax)
export(get_skewness)
export(get_start_vals)
export(get_thermalsafetymargin)
export(get_thermaltolerance)
export(get_topt)
export(get_tpc_as_formula)
export(get_upper_lims)
export(hinshelwood_1947)
export(janisch1_1925)
export(janisch2_1925)
export(joehnk_2008)
export(johnsonlewin_1946)
export(kamykowski_1985)
export(lactin2_1995)
export(lobry_1991)
export(lrf_1991)
export(mitchell_2009)
export(modifiedgaussian_2006)
export(oneill_1972)
export(pawar_2018)
export(quadratic_2008)
export(quickfit_tpc)
export(quickfit_tpc_multi)
export(ratkowsky_1983)
export(rezende_2019)
export(rosso_1993)
export(sharpeschoolfull_1981)
export(sharpeschoolhigh_1981)
export(sharpeschoollow_1981)
export(spain_1982)
export(stinner_1974)
export(taylorsexton_1972)
export(thomas_2012)
export(thomas_2017)
export(tomlinsonphillips_2015)
export(warrendreyer_2006)
export(weibull_1995)
